import {has} from 'lodash'
import CodeMirror from 'codemirror/lib/codemirror'

// loading modes
import 'codemirror/mode/css/css'
import 'codemirror/mode/htmlmixed/htmlmixed'

// styles
import 'codemirror/lib/codemirror.css'

// themes
import 'codemirror/theme/eclipse.css'

// addons
import 'codemirror/addon/selection/active-line'
import 'codemirror/addon/edit/matchbrackets'

import './styles.css'

const DEFAULT_MODE = 'htmlmixed'
const DEFAULT_THEME = 'eclipse'
const DEFAULT_TAB_SIZE = 2

function config(opts = {}) {
  has(opts, 'lineNumbers') || (opts.lineNumbers = true)
  has(opts, 'tabSize') || (opts.tabSize = DEFAULT_TAB_SIZE)
  has(opts, 'styleActiveLine') || (opts.styleActiveLine = true)
  has(opts, 'matchBrackets') || (opts.matchBrackets = true)
  has(opts, 'mode') || (opts.mode = DEFAULT_MODE)
  has(opts, 'theme') || (opts.theme = DEFAULT_THEME)
  return opts
}

function enforceMaxLength(max_length) {
  return function(cm, change) {
    if (!change.update)
      return true

    let str = change.text.join("\n")
    let delta = str.length - (cm.indexFromPos(change.to) - cm.indexFromPos(change.from))

    if (delta <= 0)
      return true

    delta = cm.getValue().length + delta - max_length

    if (delta > 0) {
      str = str.substr(0, str.length - delta)
      change.update(change.from, change.to, str.split("\n"))
    }

    return true
  }
}

function self(el, opts = {}) {

  let editor = CodeMirror.fromTextArea(el, config(opts))
  editor.on('blur', () => editor.save())

  if (opts.max_length)
    editor.on('beforeChange', enforceMaxLength(opts.max_length))

  setTimeout(() => {editor.refresh()}, 100)

  return editor
}

self.api = CodeMirror
export default self
